<?php
/**
 * This filter provides automatic support for KaTeX
 *
 * @package    filter_katexloader
 * @copyright  2018 Kosaku Nagasaka (nagasaka@main.h.kobe-u.ac.jp)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

/**
 * KaTeX filtering
 */
class filter_katexloader extends moodle_text_filter {

    /*
     * Add the javascript to enable katex processing on this page.
     *
     * @param moodle_page $page The current page.
     * @param context $context The current context.
     */
    public function setup($page, $context) {
        // This only requires execution once per request.
        static $jsinitialised = false;

        if (empty($jsinitialised)) {
            $katex_min_css_url = get_config('filter_katexloader','katex_min_css_url');
            $katex_min_js_url = get_config('filter_katexloader','katex_min_js_url');
            $katex_min_js_url = preg_replace('/\.js$/', '', $katex_min_js_url);
            $autorender_min_js_url = get_config('filter_katexloader','autorender_min_js_url');
            $autorender_min_js_url = preg_replace('/\.js$/', '', $autorender_min_js_url);

            $katex_amd_config = '"katex": "' . $katex_min_js_url . '"';
            $autorender_amd_config = '"renderMathInElement": "' . $autorender_min_js_url . '"';

            $amd_config = 'require.config({paths: {' . $katex_amd_config . ',' . $autorender_amd_config . '},waitSeconds: 40});';
            $amd_init = 'require(["katex"],function(katex){window.katex=katex;require(["renderMathInElement"],function(renderMathInElement){window.renderMathInElement=renderMathInElement;});});';

            $page->requires->js_amd_inline($amd_config . $amd_init);

            $delimiters_config = get_config('filter_katexloader', 'delimiters');
            $ignored_tags_config = get_config('filter_katexloader', 'ignored_tags');
            $params = array('katex_min_js_url' => $katex_min_js_url,
                            'autorender_min_js_url' => $autorender_min_js_url,
                            'css' => $katex_min_css_url, 
                            'options' => '{delimiters: '.$delimiters_config.', ignoredTags: '.$ignored_tags_config.'}');
            $page->requires->yui_module('moodle-filter_katexloader-loader', 'M.filter_katexloader.configure', array($params));
            $jsinitialised = true;
        }
    }

    /*
     * This function wraps the filtered text in a span, that katexloader is configured to process.
     *
     * @param string $text The text to filter.
     * @param array $options The filter options.
     */
    public function filter($text, array $options = array()) {
        global $PAGE;

        $delimiters = get_config('filter_katexloader', 'delimiters');
        $raw_split_delimiters = preg_split('/"/', $delimiters);
        $delimiters = array();
        while ( count($raw_split_delimiters) > 3 ) {
           array_pop($raw_split_delimiters);
           $right_delimiter = array_pop($raw_split_delimiters);
           array_pop($raw_split_delimiters);
           $left_delimiter = array_pop($raw_split_delimiters);
           $delimiters[] = array($left_delimiter, $right_delimiter);
        }

        $hasdelimiters = false;
        foreach ($delimiters as $delimiter) {
            if ( strpos($text, $delimiter[0]) !== false && strpos($text, $delimiter[1]) !== false ) {
                $hasdelimiters = true;
                break;
            }
        }

        if ($hasdelimiters) {
            $PAGE->requires->yui_module('moodle-filter_katexloader-loader', 'M.filter_katexloader.typeset');
            // If the dilimiters are used, wrap whole text to prevent autolinking.
            $text = '<span class="nolink">' . $text . '</span>';
            return '<span class="filter_katexloader_equation">' . $text . '</span>';
        }
        return $text;
    }
}
