/**
 * KaTeX JS Loader.
 *
 * @package    filter_katexloader
 * @copyright  2018 Kosaku Nagasaka (nagasaka@main.h.kobe-u.ac.jp)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
M.filter_katexloader = M.filter_katexloader || {
    /**
     * The users options.
     * @property _options
     * @type Object
     * @default {}
     * @private
     */
    _options: {},

    /**
     * Called by the filter when it is active on any page.
     * This does not load KaTeX yet - it loads the configuration.
     * It also subscribes to the filter-content-updated event so KaTeX can respond to content loaded by Ajax.
     *
     * @method typeset
     * @param {Object} params List of configuration params.
     */
    configure: function(params) {
        var css = document.createElement("link");
        css.rel = 'stylesheet';
        css.href = params.css;
        document.getElementsByTagName("head")[0].appendChild(css);
        // Save the options for KaTeX.
        this._options = params.options;
        // AMD
        require.config(
            {
                paths: { 
                    "katex" : params.katex_min_js_url,
                    "renderMathInElement" : params.autorender_min_js_url
                }, 
                waitSeconds: 40
            });
        // Listen for events triggered when new text is added to a page that needs
        // processing by a filter.
        Y.on(M.core.event.FILTER_CONTENT_UPDATED, this.contentUpdated, this);
    },

    /**
     * Called by the filter when an equation is found while rendering the page.
     *
     * @method typeset
     */
    typeset: function() {
        var self = this;
        if (typeof window.renderMathInElement === "undefined") {
            require(["katex"],function(katex) {
                window.katex=katex;
                require(["renderMathInElement"],function(renderMathInElement) {
                    window.renderMathInElement=renderMathInElement;
                    Y.all('.filter_katexloader_equation').each(function(node) {
                        window.renderMathInElement(node.getDOMNode(), self._options);
                    });
                });
            });
        } else {
            Y.all('.filter_katexloader_equation').each(function(node) {
                window.renderMathInElement(node.getDOMNode(), self._options);
            });
        }
    },

    /**
     * Handle content updated events - typeset the new content.
     * @method contentUpdated
     * @param Y.Event - Custom event with "nodes" indicating the root of the updated nodes.
     */
    contentUpdated: function(event) {
        var self = this;
        if (typeof window.renderMathInElement === "undefined") {
            require(["katex"],function(katex) {
                window.katex=katex;
                require(["renderMathInElement"],function(renderMathInElement) {
                    window.renderMathInElement=renderMathInElement;
                    event.nodes.each(function (node) {
                        node.all('.filter_katexloader_equation').each(function(node) {
                            window.renderMathInElement(node.getDOMNode(), self._options);
                        });
                    });
                });
            });
        } else {
            event.nodes.each(function (node) {
                node.all('.filter_katexloader_equation').each(function(node) {
                    window.renderMathInElement(node.getDOMNode(), self._options);
                });
            });
        }
    }
};

